/*
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Adrian Szyndela <adrian.s@samsung.com>
 *          Łukasz Stelmach <l.stelmach@samsung.com>
 */
/*
 * @file crash-stack.h
 * @brief unwinding call stacks, header file for common functions
 */
#ifndef CRASH_STACK_H
#define CRASH_STACK_H

#include <stdint.h>
#include <elfutils/libdwfl.h>

/**
 * @brief callstack.procedure info
 */
struct ProcInfo {
	uintptr_t addr; ///< procedure address
	int offset; ///< procedure offset
	char *name; ///< procedure name
	char *module_name; ///< module name
	int module_offset; ///< symbol's offset in the module
};
typedef struct ProcInfo ProcInfo;

#define MAX_CALLSTACK_LEN 1000
/**
 * @brief call stack database
 */
struct Callstack {
	ProcInfo proc[MAX_CALLSTACK_LEN]; ///< storage for elements
	size_t elems;						///< number of elements in the database
};
typedef struct Callstack Callstack;

/**
 * @brief Gets address of storage for specified register value
 *
 * @remarks This function uses static storage. Thus, it shouldn't be used in multiple threads.
 *
 * @param regname name of the register, e.g. "pc", "eax", "r7"
 * @param regnum ordinary number of the register, when name is not available
 * @return address of statically allocated storage of proper size for the register value
 */
void *_get_place_for_register_value(const char *regname, int regnum);

/**
 * @brief Creates call stack database.
 *
 * @param dwfl dwfl handle
 * @param pid pid of the analyzed process
 * @param callstack pointer to call stack database; results are put here
 */
void _create_crash_stack(Dwfl *dwfl, pid_t pid, Callstack *callstack);

/**
 * @brief Gets allocated space for getting target registers with ptrace().
 *
 * @remarks This function uses static storage. Thus, it shouldn't be used in multiple threads.
 *
 * @param[out] size size of the registers structure
 * @return address of statically allocated storage of proper size for the registers
 */
void *_crash_stack_get_memory_for_ptrace_registers(size_t *size);

/**
 * @brief Sets current values of registers from the ptrace buffer
 *
 * @param regbuf buffer used for getting registers with ptrace()
 *
 * @see _crash_stack_get_memory_for_ptrace_registers()
 */
void _crash_stack_set_ptrace_registers(void *regbuf);

/**
 * @brief Print values of registers from the ptrace buffer
 *
 * @param outputfile output stream
 */
void _crash_stack_print_regs(FILE* outputfile);
#endif /* CRASH_STACK_H */
